@@ -1,11 +1,9 @@
 import click
-import json
 
-from gretel_client.cli.common import SessionContext, pass_session
-from gretel_client.projects.projects import search_projects
+from gretel_client.cli.common import pass_session, SessionContext
+from gretel_client.config import write_config
 from gretel_client.projects import create_project
-from gretel_client.rest.exceptions import ApiException
-from gretel_client.config import GretelClientConfigurationError, write_config
+from gretel_client.projects.projects import get_project, search_projects
 
 
 @click.group(help="Commands for working with Gretel projects.")
@@ -13,7 +11,7 @@ def projects():
     ...
 
 
-@projects.command()
+@projects.command(help="Create a new project.")
 @click.option("--name", metavar="name", help="Gretel project name.")
 @click.option("--desc", metavar="description", help="Gretel project description.")
 @click.option(
@@ -29,30 +27,10 @@ def projects():
 def create(
     sc: SessionContext, name: str, desc: str, display_name: str, set_default: bool
 ):
-    try:
-        project = create_project(
-            name=name, desc=desc, display_name=display_name
-        )
-    except ApiException as ex:
-        if ex.status == 400:
-            error_dict = json.loads(ex.body)
-            error_msg = error_dict.get("message")
-            sc.log.error(
-                (
-                    f"Error creating project: {error_msg}"
-                ),
-                ex=ex,
-            )
-            sc.exit(1)
-        elif ex.status == 401:
-            sc.log.error("Unauthorized. Please check your credentials.")
-            sc.exit(1)
-        else:
-            sc.log.error(f"HTTP Error: {ex.body}")
-            sc.exit(1)
+    project = create_project(name=name, desc=desc, display_name=display_name)
 
     sc.log.info(f"Created project {project.name}.")
-    sc.log.info(f"Console link: {project.get_console_url()}")
+    sc.log.info(f"Console link: {project.get_console_url()}.")
 
     if set_default:
         sc.config.default_project_name = project.name
@@ -62,7 +40,7 @@ def create(
     sc.print(data=project.info())
 
 
-@projects.command()
+@projects.command(help="Search for projects.")
 @click.option("--limit", help="Limit the number of projects.", default=200)
 @click.option("--query", help="Filter project names by a query string.", default=None)
 @pass_session
@@ -72,7 +50,7 @@ def search(sc: SessionContext, limit: int, query: str):
     sc.print(data=projects_table)
 
 
-@projects.command()
+@projects.command(help="Set default project.")
 @click.option(
     "--name",
     metavar="project-name",
@@ -81,13 +59,36 @@ def search(sc: SessionContext, limit: int, query: str):
 )
 @pass_session
 def set_default(sc: SessionContext, name: str):
-    try:
-        sc.config.update_default_project(name)
-    except GretelClientConfigurationError as ex:
-        sc.log.error(f"Project {name} does not exist.", ex=ex)
-        sc.exit(1)
-
+    sc.config.update_default_project(name)
     write_config(sc.config)
-
     sc.log.info(f"Set {name} as the default project.")
     sc.print(data=sc.config.masked)
+
+
+@projects.command(help="Delete project.")
+@click.option(
+    "--name",
+    metavar="project-name",
+    help="Gretel project name, mutually exclusive with id.",
+    default=None,
+)
+@click.option(
+    "--uid",
+    metavar="project-id",
+    help="Gretel project id, mutually exclusive with name.",
+    default=None,
+)
+@pass_session
+def delete(sc: SessionContext, name: str, uid: str):
+    if name and uid:
+        raise click.BadOptionUsage(
+            "--uid",
+            f"Cannot pass both --uid and --name. Please use --name or --uid option.",
+        )
+    if name or uid:
+        project = get_project(name=name or uid)
+    else:
+        raise click.BadOptionUsage("--name", "Please use --name or --uid option.")
+    sc.print(data=project.as_dict)
+    project.delete()
+    sc.log.info("Project was deleted.")
