@@ -1,11 +1,13 @@
 from pathlib import Path
 from typing import Callable
 
+import pandas as pd
 import pytest
 import smart_open
+
+from gretel_client.cli.utils.parser_utils import RefData
 from gretel_client.config import RunnerMode
 from gretel_client.helpers import poll
-
 from gretel_client.projects.common import ModelRunArtifact
 from gretel_client.projects.docker import ContainerRun
 from gretel_client.projects.jobs import Job, Status, WaitTimeExceeded
@@ -38,16 +40,44 @@ def test_does_get_model_from_id(project: Project, transform_model_path: Path):
     assert model_remote.status
 
 
+def test_create_model_upload_artifacts(
+    project: Project,
+    transform_model_path: Path,
+    transform_local_data_source: Path,
+):
+    data_refs = [
+        {"foo": str(transform_local_data_source)},
+        str(transform_local_data_source),
+        [str(transform_local_data_source)],
+        pd.DataFrame([{"test_key": "test_value"}]),
+    ]
+
+    for ref_data in data_refs:
+        model: Model = project.create_model_obj(
+            transform_model_path,
+            data_source=str(transform_local_data_source),
+            ref_data=ref_data,
+        )
+        model.upload_data_source()
+        model.upload_ref_data()
+        assert model.data_source.startswith("gretel_")
+        assert model.ref_data.values[0].startswith("gretel_")
+
+
 def test_does_upload_local_artifact(
     project: Project, transform_model_path: Path, transform_local_data_source: Path
 ):
     ds = str(transform_local_data_source)
     m = Model(project=project, model_config=transform_model_path)
     m.data_source = ds
+    m.ref_data = RefData.from_list([ds])
     assert m.data_source == ds
     assert m.model_config["models"][0][m.model_type]["data_source"] == ds
+    assert m.model_config["models"][0][m.model_type]["ref_data"].get(0) == ds
     m.upload_data_source()
+    m.upload_ref_data()
     assert m.data_source.startswith("gretel_")
+    assert m.ref_data.values[0].startswith("gretel_")
 
 
 def test_does_train_model_and_transform_records(
@@ -56,18 +86,14 @@ def test_does_train_model_and_transform_records(
     transform_local_data_source: Path,
 ):
     m = Model(project=project, model_config=transform_model_path)
-    m.submit(runner_mode=RunnerMode.CLOUD)
+    m.submit_cloud()
     logs = list(m.poll_logs_status())
     assert len(logs) > 1
     assert m.status == Status.COMPLETED
-    record_handler = m.create_record_handler_obj()
-    record_handler.submit(
-        params=None,
-        action="transform",
-        runner_mode=RunnerMode.CLOUD,
-        data_source=str(transform_local_data_source),
-        upload_data_source=True,
+    record_handler = m.create_record_handler_obj(
+        data_source=str(transform_local_data_source)
     )
+    record_handler.submit_cloud()
     logs = list(record_handler.poll_logs_status())
     assert len(logs) > 1
     assert record_handler.status == Status.COMPLETED
@@ -77,7 +103,6 @@ def test_raises_wait_time_exceeded(
     project: Project,
     transform_model_path: Path,
 ):
-    ...
     m = Model(project=project, model_config=transform_model_path)
     m.submit(runner_mode=RunnerMode.CLOUD)
 
@@ -92,11 +117,10 @@ def test_raises_wait_time_exceeded(
     assert m.status == Status.COMPLETED
 
 
-def test_does_get_synthetic_records(trained_synth_model: Model, request):
-    handler = trained_synth_model.create_record_handler_obj()
-    handler.submit(
-        action="generate", runner_mode=RunnerMode.CLOUD, params={"num_records": 100}
-    )
+def test_does_get_records(trained_synth_model: Model, get_fixture: Callable, request):
+    handler = trained_synth_model.create_record_handler_obj(params={"num_records": 10})
+    handler.submit_cloud()
+
     request.addfinalizer(handler.delete)
     logs = list(handler.poll_logs_status())
     assert handler
@@ -116,18 +140,18 @@ def test_polls_with_helper(
     capsys,
 ):
     m = Model(project=project, model_config=transform_model_path)
-    m.submit(runner_mode=RunnerMode.CLOUD)
+    m.submit_cloud()
     poll(m)
     captured = capsys.readouterr()
     assert "Model creation complete" in captured.err
     assert m.status == Status.COMPLETED
 
 
-def test_does_search_models(pre_trained_project: Project):
-    models = list(pre_trained_project.search_models())
+def test_does_search_models(pretrained_project: Project):
+    models = list(pretrained_project.search_models())
     assert len(models) > 0
     assert all([isinstance(m, Model) for m in models])
 
-    models = list(pre_trained_project.search_models(factory=dict))
+    models = list(pretrained_project.search_models(factory=dict))
     assert len(models) > 0
     assert all([isinstance(m, dict) for m in models])
