@@ -1,23 +1,25 @@
 import os
-from pathlib import Path
-from unittest.mock import MagicMock, patch
+
 from typing import Callable
+from unittest.mock import ANY, MagicMock, patch
 
 import pytest
+
 from click.testing import CliRunner
 
 from gretel_client.cli.cli import cli
-from gretel_client.cli.common import ModelObjectReader, download_artifacts
+from gretel_client.cli.common import ModelObjectReader
+from gretel_client.cli.utils.parser_utils import RefData
 from gretel_client.config import (
     ClientConfig,
+    configure_session,
+    get_session_config,
     GRETEL_API_KEY,
     GRETEL_CONFIG_FILE,
     GRETEL_ENDPOINT,
     GRETEL_PROJECT,
-    configure_session,
-    get_session_config,
 )
-from gretel_client.projects.docker import DockerEnvironmentError
+from gretel_client.projects.exceptions import DockerEnvironmentError
 from gretel_client.projects.jobs import Status
 
 
@@ -32,7 +34,7 @@ def runner() -> CliRunner:
 @patch("gretel_client.cli.cli.write_config")
 def test_configure_env(write_config: MagicMock, runner: CliRunner):
     orig_api, orig_proj, orig_endpoint = "orig_api", "orig_proj", "orig_endpoint"
-    new_api, new_proj, new_endpoint = "new_api", "new_proj", "new_endpoint"
+    new_api, new_proj, new_endpoint = "new_api", "new_proj", "http:/new_endpoint"
 
     with patch.dict(
         os.environ,
@@ -61,8 +63,9 @@ def test_configure_env(write_config: MagicMock, runner: CliRunner):
         )
 
     assert get_session_config().api_key == new_api
-    assert get_session_config().endpoint == new_endpoint
+    assert get_session_config().endpoint == "https://new_endpoint"
     assert get_session_config().default_project_name == new_proj
+    assert not get_session_config().preview_features_enabled
     assert cmd.exit_code == 0
 
 
@@ -136,20 +139,6 @@ def test_local_model_upload_disabled_by_default(
     assert container_run.from_job.return_value.enable_cloud_uploads.call_count == 0
 
 
-def test_does_write_artifacts_to_disk(tmpdir: Path, get_fixture: Callable):
-    sc = MagicMock()
-    job = MagicMock()
-    base_endpoint = (
-        "https://gretel-public-website.s3.us-west-2.amazonaws.com/tests/client/"
-    )
-    files = ["account-balances.csv", "report_json.json.gz", "model.tar.gz"]
-    keys = ["data", "report_json", "model"]
-    job.get_artifacts.return_value = iter(zip(keys, [base_endpoint + f for f in files]))
-    download_artifacts(sc, str(tmpdir), job)
-    for file in files:
-        assert (tmpdir / file).exists()
-
-
 def test_does_read_model_json(runner: CliRunner, get_fixture: Callable):
     output = get_fixture("xf_model_create_output.json")
     sc = MagicMock()
@@ -200,3 +189,93 @@ def test_does_gracefully_handler_docker_errors(
     check_docker_env.assert_called_once()
     assert cmd.exit_code == 1
     assert "docker is not running" in cmd.output
+
+
+def test_can_name_model(get_project: MagicMock, runner: CliRunner):
+    cmd = runner.invoke(
+        cli,
+        [
+            "models",
+            "create",
+            "--config",
+            "synthetics/default",
+            "--name",
+            "test-project",
+        ],
+    )
+    assert cmd.exit_code == 0
+    assert get_project.return_value.create_model_obj.return_value.name == "test-project"
+
+
+@patch("gretel_client.cli.records.create_and_run_record_handler")
+def test_does_pass_through_manual_artifacts(
+    create_record_handler: MagicMock, get_project: MagicMock, runner: CliRunner
+):
+    cmd = runner.invoke(
+        cli,
+        [
+            "records",
+            "transform",
+            "--model-id",
+            "test_model_id",
+            "--in-data",
+            "s3://test/object.csv",
+            "--runner",
+            "manual",
+        ],
+    )
+    print(cmd.output)
+    assert cmd.exit_code == 0
+    create_record_handler.assert_called_once_with(
+        ANY,
+        params=None,
+        data_source="s3://test/object.csv",
+        action="transform",
+        runner="manual",
+        output=None,
+        in_data="s3://test/object.csv",
+        status_strings=ANY,
+        model_path=None,
+    )
+
+
+@patch("gretel_client.cli.records.create_and_run_record_handler")
+def test_does_run_manual_artifacts(
+    create_record_handler: MagicMock, get_project: MagicMock, runner: CliRunner
+):
+    cmd = runner.invoke(
+        cli,
+        [
+            "models",
+            "run",
+            "--model-id",
+            "test_model_id",
+            "--in-data",
+            "s3://test/object.csv",
+            "--ref-data",
+            "gcs://test/test-data.csv",
+            "--ref-data",
+            "azure://test.csv",
+            "--runner",
+            "manual",
+        ],
+    )
+    print(cmd.output)
+    assert cmd.exit_code == 0
+    create_record_handler.assert_called_once_with(
+        ANY,
+        params=None,
+        action=None,
+        in_data="s3://test/object.csv",
+        data_source="s3://test/object.csv",
+        ref_data=RefData(
+            ref_dict={0: "gcs://test/test-data.csv", 1: "azure://test.csv"}
+        ),
+        config_ref_data=RefData(
+            ref_dict={0: "gcs://test/test-data.csv", 1: "azure://test.csv"}
+        ),
+        runner="manual",
+        output=None,
+        status_strings=ANY,
+        model_path=None,
+    )
